// SPDX-FileCopyrightText: OpenTalk GmbH <mail@opentalk.eu>
//
// SPDX-License-Identifier: EUPL-1.2
import { styled, FormHelperText, Button, Grid } from '@mui/material';
import { Seconds } from '@opentalk/common';
import { BackIcon, NoOfParticipantsIcon, NoOfRoomsIcon } from '@opentalk/common';
import { notifications } from '@opentalk/common';
import { FormikProps } from 'formik';
import { Step, FormikWizard } from 'formik-wizard-form';
import { FormikValues } from 'formik/dist/types';
import i18next from 'i18next';
import { get, shuffle } from 'lodash';
import React, { useCallback, useRef } from 'react';
import { useTranslation } from 'react-i18next';
import * as Yup from 'yup';

import { start } from '../../../api/types/outgoing/breakout';
import AccordionItem from '../../../commonComponents/AccordionItem';
import { useAppDispatch, useAppSelector } from '../../../hooks';
import { selectCombinedParticipantsAndUser, selectCombinedParticipantsAndUserCount } from '../../../store/selectors';
import { Participant } from '../../../store/slices/participantsSlice';
import { spliceIntoChunks } from '../../../utils/arrayUtils';
import CreateByParticipantsForm from './CreateByParticipantsForm';
import CreateByRoomsForm from './CreateByRoomsForm';
import ParticipantsSelector, { BreakoutRoomWithFullParticipants } from './ParticipantsSelector';

export enum AccordionOptions {
  Rooms = 'rooms',
  Participants = 'participants',
  Groups = 'groups',
  Moderators = 'Moderators',
}

type Expanded = string | false;

const Form = styled('form')({
  flex: 1,
});

class HandleSubmit extends React.Component<{ handleNext: () => void; formik: FormikProps<FormikValues> }> {
  componentDidMount() {
    if (this.props.formik.submitCount === 0) {
      this.props.handleNext();
    }
  }
  render() {
    return null;
  }
}

const validationSchema = () =>
  Yup.object({
    expanded: Yup.mixed<AccordionOptions>()
      .oneOf(Object.values(AccordionOptions), i18next.t('breakout-room-form-error-expanded'))
      .required(i18next.t('breakout-room-form-error-expanded')),
    rooms: Yup.object().when(['expanded'], {
      is: AccordionOptions.Rooms,
      then: Yup.object({
        rooms: Yup.number().min(2, i18next.t('breakout-room-form-error-min-room')),
      }),
    }),
    participants: Yup.object().when(['expanded'], {
      is: AccordionOptions.Participants,
      then: Yup.object().shape({
        participantsPerRoom: Yup.number().min(2, i18next.t('breakout-room-form-error-min-participants')),
      }),
    }),
  });

const CreateRoomsForm = () => {
  const [expanded, setExpanded] = React.useState<Expanded>(false);
  const dispatch = useAppDispatch();
  const participantsTotal = useAppSelector(selectCombinedParticipantsAndUserCount);
  const participants = useAppSelector(selectCombinedParticipantsAndUser);
  const insufficientParticipants = participantsTotal < 4;
  const { t } = useTranslation();
  const handleNextRef = useRef<() => void>();
  const handlePrevRef = useRef<() => void>();

  const handleNext = useCallback(() => {
    if (insufficientParticipants) {
      notifications.error(t('breakout-room-create-button-disabled'));
      return;
    }

    handleNextRef.current?.();
  }, [insufficientParticipants, t]);

  const handleChange =
    (panel: string, formik: FormikProps<FormikValues>) => (event: React.ChangeEvent<unknown>, newExpanded: boolean) => {
      const expanded = newExpanded ? panel : false;
      setExpanded(expanded);
      formik.setFieldValue('expanded', expanded);
      formik.setErrors({});
    };

  const handleStepBackOnParticipantSelector = (formik: FormikProps<FormikValues>) => {
    handlePrevRef.current?.();
    formik.setFieldValue(`${formik.values.expanded}.assignments`, []);
  };

  const steps: Step[] = React.useMemo(
    () => [
      {
        component: (formik) => (
          <>
            <AccordionItem
              onChange={handleChange(AccordionOptions.Rooms, formik)}
              option={AccordionOptions.Rooms}
              expanded={expanded === AccordionOptions.Rooms}
              summaryText={t('breakout-room-tab-by-rooms')}
              summaryIcon={<NoOfRoomsIcon />}
            >
              <CreateByRoomsForm formName={AccordionOptions.Rooms} handleNext={handleNext} />
            </AccordionItem>
            <AccordionItem
              onChange={handleChange(AccordionOptions.Participants, formik)}
              option={AccordionOptions.Participants}
              expanded={expanded === AccordionOptions.Participants}
              summaryText={t('breakout-room-tab-by-participants')}
              summaryIcon={<NoOfParticipantsIcon />}
            >
              <CreateByParticipantsForm formName={AccordionOptions.Participants} handleNext={handleNext} />
            </AccordionItem>
            {/*
            todo add if the information about groups and moderators is available

            <AccordionItem
            onChange={handleChange(AccordionOptions.Groups, formik)}
            option={AccordionOptions.Groups}
            expanded={expanded === AccordionOptions.Groups}
            summaryText={t('breakout-room-tab-by-groups')}
            summaryIcon={<GroupsIcon />}
          >
            <CreateByGroupsForm
              formName={AccordionOptions.Groups}
              formik={formik}
              handleNext={handleNextRef.current!}
            />
          </AccordionItem>
          <AccordionItem
            onChange={handleChange(AccordionOptions.Moderators, formik)}
            option={AccordionOptions.Moderators}
            expanded={expanded === AccordionOptions.Moderators}
            summaryText={t('breakout-room-tab-by-moderators')}
            summaryIcon={<ModeratorIcon />}
          >
            <CreateByModeratorsForm
              formName={AccordionOptions.Moderators}
              formik={formik}
              handleNext={handleNextRef.current!}
            />
          </AccordionItem>
            */}
            {formik.errors.expanded && <FormHelperText error>{t(formik.errors.expanded as string)}</FormHelperText>}
          </>
        ),
        validationSchema: validationSchema(),
      },
      {
        component: (formik) => {
          const formState = get(formik.values, formik.values.expanded, undefined);
          return formState.distribution ? (
            <HandleSubmit handleNext={handleNextRef.current as () => void} formik={formik} />
          ) : (
            <Grid container spacing={1} direction={'column'}>
              <Grid item xs>
                <Button
                  variant={'text'}
                  size={'small'}
                  startIcon={<BackIcon />}
                  onClick={() => handleStepBackOnParticipantSelector(formik)}
                >
                  {t('user-selection-button-back')}
                </Button>
              </Grid>
              <Grid item xs>
                <ParticipantsSelector
                  onSubmit={handleNextRef.current as () => void}
                  formName={formik.values.expanded}
                  name={`${formik.values.expanded}.assignments`}
                />
              </Grid>
            </Grid>
          );
        },
      },
    ],
    [expanded, t, handleNext]
  );

  const generateRandomAssignments = (rooms: number) => {
    const shuffledParticipants = shuffle(participants);
    const chunkedParticipants: Array<Array<Participant>> = spliceIntoChunks(shuffledParticipants, rooms);

    const breakoutRoomAssignments: BreakoutRoomWithFullParticipants[] = chunkedParticipants.map(
      (participants, index) => ({
        name: t('room', { roomNumber: index + 1 }),
        assignments: participants,
      })
    );
    return breakoutRoomAssignments;
  };

  const mapParticipantsToParticipantIds = (participants: Participant[]) =>
    participants.map((participant) => participant.id);

  const startByRooms = (values: FormikValues) => {
    const { duration, rooms, assignments, distribution } = get(values, values.expanded, undefined);
    let assignedParticipants = assignments as BreakoutRoomWithFullParticipants[];
    if (distribution) {
      assignedParticipants = generateRandomAssignments(rooms);
    }

    dispatch(
      start.action({
        duration: duration ? ((duration * 60) as Seconds) : undefined,
        strategy: 'manual',
        rooms: assignedParticipants.map((breakoutRoom) => ({
          ...breakoutRoom,
          assignments: mapParticipantsToParticipantIds(breakoutRoom.assignments),
        })),
      })
    );
  };

  const startByParticipants = (values: FormikValues) => {
    const { duration, participantsPerRoom, assignments, distribution } = get(values, values.expanded, undefined);
    const rooms = Math.floor(participantsTotal / participantsPerRoom);
    let assignedParticipants = assignments as BreakoutRoomWithFullParticipants[];
    if (distribution) {
      assignedParticipants = generateRandomAssignments(rooms);
    }

    dispatch(
      start.action({
        duration: duration ? ((duration * 60) as Seconds) : undefined,
        strategy: 'manual',
        rooms: assignedParticipants.map((breakoutRoom) => ({
          ...breakoutRoom,
          assignments: mapParticipantsToParticipantIds(breakoutRoom.assignments),
        })),
      })
    );
  };

  // todo include/exclude moderators
  const handleSubmit = (values: FormikValues) => {
    switch (values.expanded) {
      case AccordionOptions.Rooms:
        startByRooms(values);
        break;
      case AccordionOptions.Participants:
        startByParticipants(values);
        break;
      case AccordionOptions.Groups:
      case AccordionOptions.Moderators:
      default:
    }
  };

  return (
    <FormikWizard
      initialValues={{
        rooms: {
          duration: undefined,
          rooms: 2,
          distribution: false,
          includeModerators: false,
          assignments: [],
        },
        participants: {
          duration: undefined,
          participantsPerRoom: 2,
          distribution: false,
          includeModerators: false,
          assignments: [],
        },
        groups: {
          duration: undefined,
          includeModerators: false,
        },
        moderators: {
          duration: undefined,
          distribution: false,
          includeModerators: true,
          assignments: [],
        },
      }}
      onSubmit={handleSubmit}
      validateOnNext
      activeStepIndex={0}
      steps={steps}
      validateOnBlur={false}
      validateOnChange={false}
    >
      {({ renderComponent, handlePrev, handleNext }) => {
        handleNextRef.current = handleNext;
        handlePrevRef.current = handlePrev;
        return <Form>{renderComponent()}</Form>;
      }}
    </FormikWizard>
  );
};

export default CreateRoomsForm;
