// SPDX-FileCopyrightText: OpenTalk GmbH <mail@opentalk.eu>
//
// SPDX-License-Identifier: EUPL-1.2
import {
  Accordion as MuiAccordion,
  AccordionDetails,
  Button as MuiButton,
  List,
  styled,
  Typography,
  AccordionSummary as MuiAccordionSummary,
  Stack,
} from '@mui/material';
import { BreakoutRoomId, ParticipationKind } from '@opentalk/common';
import { ArrowDownIcon } from '@opentalk/common';
import React, { useCallback } from 'react';
import { useTranslation } from 'react-i18next';

import { useAppSelector } from '../../../hooks';
import { selectBreakoutRoomById, selectCurrentBreakoutRoomId } from '../../../store/slices/breakoutSlice';
import { Participant } from '../../../store/slices/participantsSlice';
import { selectOurUuid } from '../../../store/slices/userSlice';
import ParticipantAvatar from '../../ParticipantAvatar';

const Avatar = styled(ParticipantAvatar)({
  width: '2.25rem',
  height: '2.25rem',
  fontSize: '0.75rem',
});

const Accordion = styled(MuiAccordion)({
  margin: '0 !important',
  backgroundColor: 'transparent',
});

const AccordionSummary = styled(MuiAccordionSummary)(({ theme }) => ({
  margin: 0,
  padding: 0,
  flexDirection: 'row-reverse',
  maxHeight: '2rem !important',
  minHeight: '2rem !important',
  '& .MuiAccordionSummary-content.Mui-expanded': {
    marginLeft: theme.spacing(1),
  },
  '& .MuiAccordionSummary-content': {
    marginLeft: theme.spacing(1),
    justifyContent: 'space-between',
    alignItems: 'center',
  },
  '& .MuiIconButton-edgeEnd': {
    marginRight: 0,
  },
  '& svg': {
    width: '0.75rem',
    color: 'white',
  },
}));

interface RoomOverviewListProps {
  joinRoom: (roomId: string) => void;
  groupedParticipants: Participant[];
  breakoutRoomId: BreakoutRoomId;
}

const RoomOverviewListItem = ({ joinRoom, groupedParticipants, breakoutRoomId }: RoomOverviewListProps) => {
  const currentBreakoutRoomId = useAppSelector(selectCurrentBreakoutRoomId);
  const breakoutRoom = useAppSelector(selectBreakoutRoomById(breakoutRoomId));
  const { t } = useTranslation();
  const ourUuid = useAppSelector(selectOurUuid);

  const getParticipantLabel = useCallback(
    (participant: Participant) => {
      if (participant?.id === ourUuid) {
        return `${participant?.displayName} ${t('breakout-room-room-overview-participant-list-me')}`;
      }
      return participant?.displayName;
    },
    [ourUuid, t]
  );

  const handleJoinRoom = (e: React.MouseEvent) => {
    e.stopPropagation();
    joinRoom(breakoutRoomId);
  };
  return (
    <Accordion defaultExpanded={true} elevation={0}>
      <AccordionSummary id={breakoutRoomId} expandIcon={<ArrowDownIcon />}>
        <Typography variant={'caption'}>{breakoutRoom?.name}</Typography>
        {currentBreakoutRoomId !== breakoutRoomId && (
          <MuiButton variant="text" size="small" onClick={handleJoinRoom}>
            {t('moderator-join-breakout-room')}
          </MuiButton>
        )}
      </AccordionSummary>
      <AccordionDetails>
        <List>
          {groupedParticipants.map((participant) => {
            return (
              <Stack spacing={1} direction={'row'} alignItems={'center'} width={'18em'} py={1} key={participant.id}>
                <Avatar
                  src={participant?.avatarUrl}
                  alt={participant?.displayName}
                  isSipParticipant={participant.participationKind === ParticipationKind.Sip}
                >
                  {participant?.displayName}
                </Avatar>
                <Typography variant={'body1'} noWrap>
                  {getParticipantLabel(participant)}
                </Typography>
              </Stack>
            );
          })}
        </List>
      </AccordionDetails>
    </Accordion>
  );
};

export default RoomOverviewListItem;
